unit hydrocal;
{$R-}    {Range checking off}
{$B+}    {Boolean complete evaluation on}
{$S+}    {Stack checking on}
{$I+}    {I/O checking on}
{$N-}    {No numeric coprocessor}


(**************************************************************
* HYDROCAL.PAS                                                *
*                                                             *
* CONTENTS                                                    *
*                                                             *
* MISCELLANEOUS HYDROLIN-PROCEDURES TU DELFT / M.J.Vos / 1988 *
***************************************************************
* HydroMat; MatPrint; VecPrint; Gauss;                        *
**************************************************************)

interface

uses crt,turbo3,types,turbolib,hydrol{,HYDROGR};

procedure gauss(var a:arrarr;var b,x:arr;var n:integer);
procedure matprint(head:str80;n,m:integer;a:arrarr);
procedure vecprint(head:str80;m:integer;y:arr);
procedure hydromat;procedure hydrocm;

implementation

(*============================================================*)
PROCEDURE Gauss(Var A:ArrArr; Var B,X:Arr; Var N:Integer);
(*============================================================*)

Var J,K,ROW,COL,MAX,JP1,NM1,KP1 :Integer;
    MUL,TEMP                    :REAL;

BEGIN
    NM1 := N-1;
    FOR J:= 1 TO NM1 DO
    BEGIN
        JP1 := J+1;
        MAX := J;
        FOR ROW := JP1 TO N DO
        BEGIN
            IF A[ROW,J] > A[MAX,J] THEN MAX:=ROW;
        END;
        FOR COL:=J TO N DO
          BEGIN
              TEMP := A[J,COL];
              A[J,COL] := A[MAX,COL];
              A[MAX,COL] := TEMP
          END;
        TEMP := B[J];
        B[J] := B[MAX];
        B[MAX] := TEMP;
        FOR K:=JP1 TO N DO
          BEGIN
            MUL := -A[K,J]/A[J,J];
            FOR COL:=J TO N DO A[K,COL]:=A[K,COL]+MUL*A[J,COL];
            B[K] := B[K] + MUL*B[J]
          END;
    END;
    X[N] := B[N]/A[N,N];
    FOR K:= NM1 DOWNTO 1 DO
      BEGIN
          TEMP := 0.0;
          KP1 := K+1;
          FOR J:= KP1 TO N DO TEMP:=TEMP+A[K,J]*X[J];
          X[K] := (B[K] - TEMP)/A[K,K]
      END;
END; (* Gauss *)

(*===========================================================*)
PROCEDURE MatPrint(Head : Str80; N,M : Integer; A : ArrArr);
(*===========================================================*)
BEGIN
 WriteLn(Head);WriteLn;
 FOR I:=1 TO M DO
 BEGIN
  FOR J:=1 TO N DO
  WriteLn(A[I,J]:12:3);
 END;
END; (* MatPrint *)

(*===========================================================*)
PROCEDURE VecPrint(Head : Str80; M : Integer; Y: Arr);
(*===========================================================*)

BEGIN
 WriteLn(Head);WriteLn;
 FOR I:=1 TO M DO
 WriteLn(Y[I]:12:3);
END; (* VecPrint *)

(*===========================================================*)
PROCEDURE Hydromat;
(*===========================================================*)

BEGIN
 Write('Give wiped nett-precipitation datafilename  ');LV;
 Write('(d:XXXXXPNX.DAT)');HV;WriteLn;
 For TEL:=1 TO MaxTEL DO PQ[TEL]:=0.0;
 Input_File;
 FOR TEL:=1 TO MaxTEL DO PQ[TEL]:=PQ[TEL]*1E-3;
 B:=PQ;P:=MaxTEL;
 WriteLn('Number of precipitation data P : ',P);

 Write('Give the extent of the area ');Delay(3000);
 Area_Define;PressKey;
 Write('Give wiped nett-discharge     datafilename  ');LV;
 Write('(d:XXXXXQWX.DAT)');HV;WriteLn;
 FOR TEL:=1 TO MaxTEL DO PQ[TEL]:=0.0;
 Input_File;
 FOR TEL:=1 TO MaxTEL DO PQ[TEL]:=PQ[TEL]/Area;
 C:=PQ;Q:=MaxTEL;
 WriteLn('Number of discharge data     Q : ',Q);WriteLn;WriteLn;

 WriteLn('Matrix MxN from : ');WriteLn;
 WriteLn('q1 = u1.i1                            ');
 WriteLn('q2 = u2.i1 + u1.i2                    ');
 WriteLn('q3 = u3.i1 + u2.i2 + u1.i3            ');
 WriteLn('etc.                                  ');
 WriteLn;WriteLn;WriteLn;
 WriteLn('i1                            u1    q1');
 WriteLn('i2  i1                        u2    q2');
 WriteLn('i3  i2  i1                    u3    q3');
 WriteLn('i4  i3  i2  i1                u4    q4');
 WriteLn('.   .   .   .   .             .     . ');
 WriteLn('in  .                    *    .  =  . ');
 WriteLn('                                    qm');
 WriteLn('etc.');PressKey;

 M:=Q;
 N:=M-P; (* Matrix-dimensions *)

 FOR I:=1 TO M DO
 BEGIN
  FOR J:=1 TO N DO
  BEGIN
   A[I,J]:=0.0E00;
  END;
 END; (* Initialisation *)


 MatPrint('INITIALISATION',N,M,A);

 FOR J:=1 TO N DO       (* Column *)
 BEGIN
  FOR I:=J TO (J+P) DO  (* Row *)
  BEGIN
   A[I,J]:=B[J];
   TA[J,I]:=A[I,J];
  END;
 END; (* MatInp *)


 MatPrint('INPUT MATRIX',N,M,A);PressKey;
 MatPrint('TRANSFORM. MATRIX',N,M,TA);PressKey;

 FOR I:=1 TO M DO
 Y[I]:=C[I]; (* Vector *)


 VecPrint('INPUT VECTOR',M,Y);PressKey;

 FOR II:=1 TO N DO
 BEGIN
  FOR I:=1 TO N DO
  BEGIN
   AAH:=0.0;
   FOR J:=1 TO M DO
   AAH:=AAH + TA[I,J] * A[J,II];
   AA[I,II]:=AAH;
  END;
 END;  (* AT*A Matrix *)


 MatPrint('AT*A MATRIX',N,M,AA);PressKey;

 FOR I:=1 TO N DO
 BEGIN
  AAH:=0.0;
  FOR J:=1 TO M DO
  AAH:=AAH + TA[I,J] * Y[J];
  AY[I]:=AAH;
 END;   (* AT*Y *)


 VecPrint('TRANSFORM.VECTOR',M,AY);PressKey;

 Gauss(AA,Y,U,N);
 WriteLn('Calculated Unit Hydrograph ordinates : ');WriteLn;
 FOR I:=1 TO M DO WriteLn('I = ',I:6,'U[',I,'] = ',U[I]:20:3);
 PressKey;

END; (* HydroMat *)
PROCEDURE HYDROCM;
BEGIN  (* HydroCal *)
 HydroMat;
 Write('Do you want to Save these data ?');LV;
 Write('  (Y/N)');HV; Read(Kbd,Ch);WriteLn;
 IF Ja THEN File_Name_Num;
 Assign(FV,FileName);
 ReWrite(FV);
 StrPQ:='UNIT DISCHARGE QUH';Strt:='TIME tUH';
 DimStrPQ:='cubic m/s';DimStrt:='hour';
 File_Text;
 FOR I:=1 TO M DO WriteLn(I:6,U[I]:20:3,t[I]:20);
 Close(FV);Scroll;ClrScr;
 WriteLn;
END; (* HydroCal *)

(*END ===================HYDROCAL.PAS =======================*)
end.