(**********************************************************************
* HYDROCAL.PAS                                                        *
*                                                                     *
* CONTENTS                                                            *
*                                                                     *
* MISCELLANEOUS HYDROLIN-PROCEDURES         TU DELFT / M.J.Vos / 1988 *
***********************************************************************
* HydroMat; MatPrint; VecPrint; Gauss;                                *
**********************************************************************)

PROGRAM HydroCal;          (* HYDROLIN version 1.00 *)

CONST
  Arr_Afm            = 30; (* Dimension precip/disch datasets *)

LABEL
  Start,Exit;

TYPE

  Str1             = STRING[1];
  Str2             = STRING[2];
  Str5             = STRING[5];
  Str6             = STRING[6];
  Str10            = STRING[10];
  Str12            = STRING[12];
  Str14            = STRING[14];
  Str20            = STRING[20];
  Str80            = STRING[80];
  Arr              = ARRAY[0..Arr_Afm] OF Real;
  ArrInt           = ARRAY[0..Arr_Afm] OF Integer;
  ArrArr           = ARRAY[1..Arr_Afm,1..Arr_Afm] OF Real;

VAR

{Hydrolin variables}

  Ch               : Char;
  Quit             : Boolean;
  FileName,FN,FN0,
  FN1,FN2          : Str14;
  Filvar,FV,FV0,FV1,
  FV2              : Text;
  Line,Line1,Line2 : Str80;
  Menu_Heading,Name,
  Titel,Method     : Str20;
  jjmmdd,Seq,Spc   : Str6;
  tt_tt            : Str5;
  Version,FileSpec : Str2;
  FNM              : Str5;
  FileNumE,FileNumR: Str1;
  PQ,rcQ,Qbr,Qbs,
  Qnett,Ibr,Iinit,
  Iphi,Inett,Iusr,
  U,Y,AY,B,C       : Arr;
  A,AA,TA          : ArrArr;
  t,tQbr,tQbs,tQnett,
  tIbr,tIinit,tIphi,
  tInett,tIusr     : ArrInt;
  tIinit1,tIinit2,
  tQs,tQu,Bf,Bpr,
  M,N,P,Q,I,II,J   : Integer;
  TEL,tTEL,fTEL,
  bTEL,sTEL,MaxTEL : Byte;
  Area,ha,sqkm,Af,
  Apr,Qs,Qu,Phi,MulP,
  Pbrtot,Qbrtot,MulQ,
  Pntot,R,Qntot,
  QT1,QT2,QT3,AAH  : Real;
  StrPQ,Strt,
  DimStrPQ,DimStrt : Str20;

{Compiler directives}
{$C+,I-,U+,R+}

{Included files: hydrolin & library}
{$I b:turbolib.inc    }
{$I b:hydrol01.inc    }
{$I b:hydrol02.inc    }


(*====================================================================*)
PROCEDURE Gauss(Var A:ArrArr; Var B,X:Arr; Var N:Integer);
(*====================================================================*)

Var J,K,ROW,COL,MAX,JP1,NM1,KP1 :Integer;
    MUL,TEMP                    :REAL;

BEGIN
    NM1 := N-1;
    FOR J:= 1 TO NM1 DO
    BEGIN
        JP1 := J+1;
        MAX := J;
        FOR ROW := JP1 TO N DO
        BEGIN
            IF A[ROW,J] > A[MAX,J] THEN MAX:=ROW;
        END;
        FOR COL:=J TO N DO
          BEGIN
              TEMP := A[J,COL];
              A[J,COL] := A[MAX,COL];
              A[MAX,COL] := TEMP
          END;
        TEMP := B[J];
        B[J] := B[MAX];
        B[MAX] := TEMP;
        FOR K:=JP1 TO N DO
          BEGIN
              MUL := -A[K,J]/A[J,J];
              FOR COL:=J TO N DO A[K,COL]:=A[K,COL]+MUL*A[J,COL];
              B[K] := B[K] + MUL*B[J]
          END;
    END;
    X[N] := B[N]/A[N,N];
    FOR K:= NM1 DOWNTO 1 DO
      BEGIN
          TEMP := 0.0;
          KP1 := K+1;
          FOR J:= KP1 TO N DO TEMP:=TEMP+A[K,J]*X[J];
          X[K] := (B[K] - TEMP)/A[K,K]
      END;
END; (* Gauss *)

(*===========================================================================*)
PROCEDURE MatPrint(Head : Str80; N,M : Integer; A : ArrArr);
(*===========================================================================*)
BEGIN
 WriteLn(Head);WriteLn;
 FOR I:=1 TO M DO
 BEGIN
  FOR J:=1 TO N DO
  WriteLn(A[I,J]:12:3);
 END;
END; (* MatPrint *)

(*===========================================================================*)
PROCEDURE VecPrint(Head : Str80; M : Integer; Y: Arr);
(*===========================================================================*)

BEGIN
 WriteLn(Head);WriteLn;
 FOR I:=1 TO M DO
 WriteLn(Y[I]:12:3);
END; (* VecPrint *)

(*===========================================================================*)
PROCEDURE Hydromat;
(*===========================================================================*)

BEGIN
 Write('Give wiped nett-precipitation datafilename  ');LV;
 Write('(d:XXXXXPNX.DAT)');HV;WriteLn;
 Input_File;
 FOR TEL:=1 TO MaxTEL DO PQ[TEL]:=PQ[TEL]*1E-3/3600;
 B:=PQ;P:=MaxTEL;
 WriteLn('Number of precipitation data P : ',P);

 Write('Give the extent of the area ');Delay(3000);
 Area_Define;PressKey;
 Write('Give wiped nett-discharge     datafilename  ');LV;
 Write('(d:XXXXXQWX.DAT)');HV;WriteLn;
 Input_File;
 FOR TEL:=1 TO MaxTEL DO PQ[TEL]:=PQ[TEL]/Area;
 C:=PQ;Q:=MaxTEL;
 WriteLn('Number of discharge data     Q : ',Q);WriteLn;WriteLn;

 WriteLn('Matrix MxN from : ');WriteLn;
 WriteLn('q1 = u1.i1                            ');
 WriteLn('q2 = u2.i1 + u1.i2                    ');
 WriteLn('q3 = u3.i1 + u2.i2 + u1.i3            ');
 WriteLn('etc.                                  ');
 WriteLn;WriteLn;WriteLn;
 WriteLn('i1                            u1    q1');
 WriteLn('i2  i1                        u2    q2');
 WriteLn('i3  i2  i1                    u3    q3');
 WriteLn('i4  i3  i2  i1                u4    q4');
 WriteLn('.   .   .   .   .             .     . ');
 WriteLn('in  .                    *    .  =  . ');
 WriteLn('                                    qm');
 WriteLn('etc.');PressKey;

 M:=Q;
 N:=M-P; (* Matrix-dimensions *)

 FOR I:=1 TO M DO
 BEGIN
  FOR J:=1 TO N DO
  BEGIN
   A[I,J]:=0.0E00;
  END;
 END; (* Initialisation *)


 MatPrint('INITIALISATION',N,M,A);

 FOR J:=1 TO N DO       (* Column *)
 BEGIN
  FOR I:=J TO (J+P) DO  (* Row *)
  BEGIN
   A[I,J]:=B[J];
   TA[J,I]:=A[I,J];
  END;
 END; (* MatInp *)


 MatPrint('INPUT MATRIX',N,M,A);PressKey;
 MatPrint('TRANSFORM. MATRIX',N,M,TA);PressKey;

 FOR I:=1 TO M DO
 Y[I]:=C[I]; (* Vector *)


 VecPrint('INPUT VECTOR',M,Y);PressKey;

 FOR II:=1 TO N DO
 BEGIN
  FOR I:=1 TO N DO
  BEGIN
   AAH:=0.0;
   FOR J:=1 TO M DO
   AAH:=AAH + TA[I,J] * A[J,II];
   AA[I,II]:=AAH;
  END;
 END;  (* AT*A Matrix *)


 MatPrint('AT*A MATRIX',N,M,AA);PressKey;

 FOR I:=1 TO N DO
 BEGIN
  AAH:=0.0;
  FOR J:=1 TO M DO
  AAH:=AAH + TA[I,J] * Y[J];
  AY[I]:=AAH;
 END;   (* AT*Y *)


 VecPrint('TRANSFORM.VECTOR',M,AY);PressKey;

 Gauss(AA,Y,U,N);
 WriteLn('Calculated Unit Hydrograph ordinates : ');WriteLn;
 FOR I:=1 TO M DO WriteLn('I = ',I:6,'U[',I,'] = ',U[I]:20:3);PressKey;

END; (* HydroMat *)

BEGIN  (* HydroCal *)
 Start:
 Scroll;ClrScr;LV;
 GotoXY(12,5);
 Write('*********************************************************');
 GotoXY(12,6);
 Write('*                                                       *');
 GotoXY(12,7);
 Write('*                                                       *');
 GotoXY(12,8);
 Write('*          HYDROLIN 1.00  TURBO  MATRIX-OPERATIONS      *');
 GotoXY(12,9);
 Write('*          -------------  -----  -----------------      *');
 GotoXY(12,10);
 Write('*                                                       *');
 GotoXY(12,11);
 Write('*                                                       *');
 GotoXY(12,12);
 Write('*        ');LV;Write(' M.J. Vos   880226    TU/DELFT/Ct');
 Write('              *');
 GotoXY(12,13);
 Write('*                                                       *');
 GotoXY(12,14);
 Write('*                                                       *');
 GotoXY(12,15);
 Write('*********************************************************');
 Delay(3000);HV;
 HydroMat;
 Write('Do you want to Save these data ?');LV;Write('  (Y/N)');HV;
 Read(Kbd,Ch);WriteLn;
 IF Ja THEN File_Name_Num;
 Assign(FV,FileName);
 ReWrite(FV);
 StrPQ:='UNIT DISCHARGE QUH';Strt:='TIME tUH';
 DimStrPQ:='m/s';DimStrt:='hour';
 File_Text;
 FOR I:=1 TO M DO WriteLn(I:6,U[I]:20:3,t[I]:20);
 Close(FV);Scroll;ClrScr;
 Write('Do you want to use GRAPHICS ? ');LV;Write(' (Y/N)');HV;Read(Kbd,Ch);
 WriteLn;
 IF Ja THEN
 BEGIN
  Assign(FV,'b:HYDROGR.CHN');
  Chain(FV);
 END ELSE
 BEGIN
  Assign(FV,'a:HYDROLIN.COM');
  Execute(FV);
 END;
END. (* HydroCal *)

(*END =======================HYDROCAL.PAS ===================================*)
